/* ---------------------------------------------------------
   Natural convection of pure water in
   a square cavity of dimensions [0,1]x[0,1]
   System of eq: Incompressible unsteady Navier-Stokes-Boussinesq
   Phase-Change: Enthalpy method h = c(T+S)
   Boyancy force: fB = Ra/(Rey^2 Pr)*1/(dT*beta)*(1-rho(T)/rho(Tf))
   Density variation: rho(T) = rhoM * ( 1 - w | T - Tm| ^ qw)
   Continuation on Temperature

               *** PHYSICAL PARAMETERS***
   --------------------------------------------------------*/

// FES and choice of the scaling -----------------
macro typeT P2// P1 or P2
macro scalAdim()1// 1,2 or 3

// Dimension of the cavity --------------------------------
real xl=0,xr=1.,yl=0,yr=1.; // Square dimension [xl,xr]x[yl,yr]
int nbseg = 40;
mesh Th=square(nbseg,nbseg);

// Dimensionless numbers ----------------------------------
real Pr=6.99,Ra=2.518084e6;
real dt=1.e-2,dtmax=1.e-1;

real Rey, IRe, IRa, IPr, Uscal, tscal;
real t,dtorig=dt;
int dtadapt=20,dtstep=10;

IFMACRO(scalAdim,1)
   Rey=1;
   Uscal = Pr/sqrt(Ra);
   tscal = 1;
ENDIFMACRO

IFMACRO(scalAdim,2)
   Rey=1./Pr;
   Uscal = 1.;
   tscal=1./Pr;
ENDIFMACRO

IFMACRO(scalAdim,3)
   Rey=sqrt(Ra/Pr);
   Uscal = sqrt(Pr);
   tscal = sqrt(Ra/Pr);
ENDIFMACRO

dt *= tscal;
dtmax *= tscal;
dtorig = dt;
real c1=1./dt,c2=-1./dt,c3=0.;

// Physical Temp ---------------------------------------
real T0hot = 10., T0cold = 0., T0fus = 0. ; // [°C]
real T0m = 4.0293;
real dTRa = T0hot-T0fus ;    // dT parameter in Ra 
real Tref = T0fus;

// Dimensionless Temp ----------------------------------
real epsi = 0.3/dTRa; // width of the mushy region
real Thot =(T0hot - Tref)/dTRa, Tcold0 =(T0cold - Tref)/dTRa;
real Tf = (T0fus - Tref)/dTRa, Tm = (T0m - Tref)/dTRa ;
int iterCT,nbcontT = 20;// nb iteration for temperature continuation
real Tcold=Tf;
real Tcoldf=Tcold,Thotf=Thot,Tin = 1.,Tinf=Tin,TcoldFreeze=Tcold0;
real Tinit = (Tcold+Thot)/2.;

// Boundary conditions ---------------------------------
int cold=2,hot=4;// label of the boundary
int[int] wall = [1,2,3,4];

macro bcu1()on(wall,uw1=0)//
macro bcu2()on(wall,uw2=0)//
macro bcT()on(cold,Tw=Tcold)+on(hot,Tw=Thot)//

IRe = 1./Rey;
IRa = Ra/(Rey*Rey*Pr); IPr = 1./(Rey*Pr);

/* Nonlinear variation of the density ------------------
  rho(T) = rhoM * ( 1 - w | T - Tm|^qw) 
  -------------------------------------------------- */
real rhoM = 999.972 ;    // [kg/m^3]
real Omega= 9.2793e-6 ;  // [deg C ^-q]
real qw = 1.894816 ;
real Betha = 6.91e-5 ;   // [K^-1]
real IBetha = 1./(Betha*dTRa);
macro TdegC(T) (T * dTRa + Tref)//

real rhoRef   = rhoM * ( 1 - Omega * (abs(Tref - T0m))^qw);
real rhoRatio = rhoM / rhoRef ;
real IrhoRatio= rhoRatio * Omega * dTRa * qw;

macro rho(T) (rhoRatio*(1 - Omega * pow(abs(TdegC(T) - T0m),qw)))//
macro Drho(T) (-IrhoRatio*signT(T)*pow(abs(TdegC(T)- T0m),qw-1))//

// Boussinesq forces ----------------------------------------
/*
   For water: fB    =   Ra/(Rey^2 Pr)*1/(dT*beta)*(1-rho(T)/rho(Tf))
                  =   -IRa * IBetha * (rho(T) - 1))
*/
macro fB(T)  (-IRa * IBetha * (rho(T) - 1))//
macro dfB(T) (-IRa * IBetha * Drho(T))//